from queue import Queue


visited = set()
come_from = {}
capacities = [10, 7, 4]


def get_neighbours(v):
    neighbours = []
    for i in range(len(v)):
        # Try to empty the jar.
        new_state = list(v)
        new_state[i] = 0
        neighbours.append(tuple(new_state))

        # Try to fill the jar.
        new_state = list(v)
        new_state[i] = capacities[i]
        neighbours.append(tuple(new_state))

        # Try to move from jar i to jar j.
        for j in range(len(v)):
            if i == j: continue
            new_state = list(v)
            new_state[j] = min(capacities[j], v[j] + v[i])
            new_state[i] = v[i] + v[j] - new_state[j]
            neighbours.append(tuple(new_state))
    return neighbours


def bfs_path(s, target):
    visited.add(tuple(s))
    toexplore = Queue()
    toexplore.put(tuple(s))

    # Traverse the graph, visiting everything reachable from s
    target_config = None
    while not toexplore.empty():
        v = toexplore.get()
        # The terminating condition is a bit different because we are searching for
        # any state that has the target jar load.
        if target in v:
            target_config = v
            break
        # Generating the neighbours is different, because we do not want to
        # construct the entire graph.
        neighbours = get_neighbours(v)
        for w in neighbours:
            if w not in visited:
                toexplore.put(w)
                visited.add(w)
                come_from[w] = v

    # Reconstruct the full path from s to t, working backwards
    if target_config is None:
        return None
    path = [target_config]
    while path[-1] in come_from:
        path.append(come_from[path[-1]])
    path.reverse()
    return path


path = bfs_path([0, 0, 0], 2)
for p in path:
    print(f"{p} -> ", end='')
print("DONE")
